import bpy
import json
import os
import tempfile
from bpy_extras.io_utils import ImportHelper, ExportHelper

bl_info = {
    "name": "Bonsai Outliner States",
    "author": "ChatGPT",
    "version": (1, 0),
    "blender": (4, 4, 0),
    "location": "3D View > Sidebar > Bonsai",
    "description": "Save and restore Outliner states per project. Supports JSON export and import.",
    "category": "3D View",
}

_cached_states = {}

# ---------------------- Functions ----------------------

def get_json_path():
    name = bpy.context.scene.get("bonsai_project_name", "default_project")
    return os.path.join(tempfile.gettempdir(), f"bonsai_visibility_states_{name}.json")

def load_states():
    global _cached_states
    path = get_json_path()
    if not path or not os.path.exists(path):
        _cached_states = {}
        return _cached_states
    try:
        with open(path, "r", encoding="utf-8") as f:
            data = json.load(f)
            if isinstance(data, dict) and data.get("type") == "bonsai_visibility_states":
                _cached_states = data.get("states", {})
            else:
                _cached_states = {}
    except:
        _cached_states = {}
    return _cached_states

def save_states(data):
    path = get_json_path()
    if not path:
        return
    try:
        full_data = {"type": "bonsai_visibility_states", "states": data}
        with open(path, "w", encoding="utf-8") as f:
            json.dump(full_data, f, indent=4)
    except Exception as e:
        print(f"❌ Error saving states: {e}")

def capture_current_state(name):
    data = load_states()
    col_data = {}
    obj_data = {}
    view_layer = bpy.context.view_layer
    layer_collections = get_all_layer_collections(view_layer.layer_collection)

    for lc in layer_collections:
        col_data[lc.collection.name] = {
            "exclude": lc.exclude,
            "hide_viewport": lc.hide_viewport,
        }

    for obj in bpy.data.objects:
        obj_data[obj.name] = {
            "hide_viewport": obj.hide_viewport,
            "hidden": obj.hide_get()
        }

    data[name] = {"collections": col_data, "objects": obj_data}
    save_states(data)

def apply_visibility_state(name):
    data = load_states().get(name)
    if not data:
        print(f"⚠️ State '{name}' not found.")
        return

    view_layer = bpy.context.view_layer
    layer_collections = get_all_layer_collections(view_layer.layer_collection)

    for lc in layer_collections:
        saved = data["collections"].get(lc.collection.name)
        if saved:
            lc.exclude = saved["exclude"]
            lc.hide_viewport = saved["hide_viewport"]

    for obj in bpy.data.objects:
        saved = data.get("objects", {}).get(obj.name)
        if saved:
            obj.hide_viewport = saved.get("hide_viewport", False)
            obj.hide_set(saved.get("hidden", False))

def delete_visibility_state(name):
    data = load_states()
    if name in data:
        del data[name]
        save_states(data)

def rename_visibility_state(old_name, new_name):
    data = load_states()
    if old_name in data:
        data[new_name] = data.pop(old_name)
        save_states(data)

def get_all_layer_collections(layer_coll):
    result = [layer_coll]
    for child in layer_coll.children:
        result.extend(get_all_layer_collections(child))
    return result

# ---------------------- Operators ----------------------

class BONSAI_OT_save_state_named(bpy.types.Operator):
    bl_idname = "bonsai.save_state_named"
    bl_label = "Save New Outliner State"

    state_name: bpy.props.StringProperty(name="Name", default="New State")

    def execute(self, context):
        capture_current_state(self.state_name)
        self.report({'INFO'}, f"Outliner state '{self.state_name}' saved.")
        return {'FINISHED'}

    def invoke(self, context, event):
        self.state_name = "New State"
        return context.window_manager.invoke_props_dialog(self)

class BONSAI_OT_apply_state_from_panel(bpy.types.Operator):
    bl_idname = "bonsai.apply_state_from_panel"
    bl_label = "Apply Outliner State"

    state_name: bpy.props.StringProperty()

    def execute(self, context):
        apply_visibility_state(self.state_name)
        self.report({'INFO'}, f"Outliner state '{self.state_name}' applied.")
        return {'FINISHED'}

class BONSAI_OT_update_state(bpy.types.Operator):
    bl_idname = "bonsai.update_state"
    bl_label = "Update Outliner State"

    state_name: bpy.props.StringProperty()

    def execute(self, context):
        capture_current_state(self.state_name)
        self.report({'INFO'}, f"Outliner state '{self.state_name}' updated.")
        return {'FINISHED'}

class BONSAI_OT_rename_state(bpy.types.Operator):
    bl_idname = "bonsai.rename_state"
    bl_label = "Rename Outliner State"

    old_name: bpy.props.StringProperty()
    new_name: bpy.props.StringProperty(name="New Name")

    def execute(self, context):
        rename_visibility_state(self.old_name, self.new_name)
        self.report({'INFO'}, f"Renamed '{self.old_name}' to '{self.new_name}'.")
        return {'FINISHED'}

    def invoke(self, context, event):
        self.new_name = self.old_name
        return context.window_manager.invoke_props_dialog(self)

class BONSAI_OT_delete_state(bpy.types.Operator):
    bl_idname = "bonsai.delete_state"
    bl_label = "Delete Outliner State"

    state_name: bpy.props.StringProperty()

    def execute(self, context):
        delete_visibility_state(self.state_name)
        self.report({'INFO'}, f"Deleted Outliner state '{self.state_name}'.")
        return {'FINISHED'}

class BONSAI_OT_save_states_as_json(bpy.types.Operator, ExportHelper):
    bl_idname = "bonsai.save_states_as_json"
    bl_label = "Save Outliner States to File"
    filename_ext = ".json"

    def execute(self, context):
        full_data = {"type": "bonsai_visibility_states", "states": _cached_states}
        try:
            with open(self.filepath, "w", encoding="utf-8") as f:
                json.dump(full_data, f, indent=2)
            self.report({'INFO'}, "Outliner states saved successfully.")
        except Exception as e:
            self.report({'ERROR'}, f"Error saving Outliner states: {e}")
        return {'FINISHED'}

class BONSAI_OT_load_states_from_json(bpy.types.Operator, ImportHelper):
    bl_idname = "bonsai.load_states_from_json"
    bl_label = "Load Outliner States from File"
    filename_ext = ".json"

    def execute(self, context):
        global _cached_states
        try:
            with open(self.filepath, "r", encoding="utf-8") as f:
                data = json.load(f)
                if isinstance(data, dict) and data.get("type") == "bonsai_visibility_states":
                    _cached_states.clear()
                    _cached_states.update(data.get("states", {}))
                    save_states(_cached_states)
                    self.report({'INFO'}, "Outliner states loaded successfully.")
                    return {'FINISHED'}
            self.report({'ERROR'}, "Invalid Outliner states file.")
            return {'CANCELLED'}
        except Exception as e:
            self.report({'ERROR'}, f"Error loading Outliner states: {e}")
            return {'CANCELLED'}

# ---------------------- Panels ----------------------

class BONSAI_PT_visibility_manager_panel(bpy.types.Panel):
    bl_label = "Bonsai Outliner States"
    bl_idname = "BONSAI_PT_visibility_manager_panel"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Bonsai"

    def draw(self, context):
        layout = self.layout
        layout.operator("bonsai.save_states_as_json", icon="EXPORT")
        layout.operator("bonsai.load_states_from_json", icon="IMPORT")
        layout.separator()
        layout.operator("bonsai.save_state_named", icon="ADD")
        layout.separator()

        states = _cached_states
        if states:
            layout.label(text="Saved States:")
            for name in sorted(states.keys()):
                row = layout.row(align=True)
                op = row.operator("bonsai.apply_state_from_panel", text=name, icon='HIDE_OFF')
                op.state_name = name
                row.operator("bonsai.update_state", text="", icon="FILE_REFRESH").state_name = name
                row.operator("bonsai.rename_state", text="", icon="GREASEPENCIL").old_name = name
                row.operator("bonsai.delete_state", text="", icon="TRASH").state_name = name
        else:
            layout.label(text="No saved Outliner states.")

# ---------------------- Registration ----------------------

classes = [
    BONSAI_PT_visibility_manager_panel,
    BONSAI_OT_save_state_named,
    BONSAI_OT_apply_state_from_panel,
    BONSAI_OT_update_state,
    BONSAI_OT_rename_state,
    BONSAI_OT_delete_state,
    BONSAI_OT_save_states_as_json,
    BONSAI_OT_load_states_from_json,
]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)

if __name__ == "__main__":
    register()
