import bpy
import json
import os
import tempfile
from bpy_extras.io_utils import ImportHelper, ExportHelper

bl_info = {
    "name": "Bonsai Viewport Bookmarks",
    "author": "ChatGPT",
    "version": (1, 1),
    "blender": (4, 4, 0),
    "location": "3D View > Sidebar > Bonsai",
    "description": "Save, restore, update, rename, and delete viewport bookmarks. Supports JSON export and import.",
    "category": "3D View",
}

_cached_views = {}

# ---------------------- Functions ----------------------

def get_view_file_path():
    name = bpy.context.scene.get("bonsai_project_name", "default_project")
    return os.path.join(tempfile.gettempdir(), f"bonsai_viewport_bookmarks_{name}.json")

def load_views():
    global _cached_views
    path = get_view_file_path()
    if not path or not os.path.exists(path):
        _cached_views = {}
        return _cached_views
    try:
        with open(path, "r", encoding="utf-8") as f:
            data = json.load(f)
            if isinstance(data, dict) and data.get("type") == "bonsai_viewport_bookmarks":
                _cached_views = data.get("views", {})
            else:
                _cached_views = {}
    except:
        _cached_views = {}
    return _cached_views

def save_views(data):
    path = get_view_file_path()
    if not path:
        return
    try:
        full_data = {"type": "bonsai_viewport_bookmarks", "views": data}
        with open(path, "w", encoding="utf-8") as f:
            json.dump(full_data, f, indent=2)
    except Exception as e:
        print(f"❌ Error saving views: {e}")

def get_region_and_space(context):
    for area in context.screen.areas:
        if area.type == 'VIEW_3D':
            for space in area.spaces:
                if space.type == 'VIEW_3D':
                    return space.region_3d, space
    return None, None

# ---------------------- Operators ----------------------

class BONSAI_OT_save_current_view(bpy.types.Operator):
    bl_idname = "bonsai.save_current_view"
    bl_label = "Save Current View"

    view_name: bpy.props.StringProperty(name="Name", default="New View")

    def execute(self, context):
        region, space = get_region_and_space(context)
        if not region:
            self.report({'ERROR'}, "Could not find 3D Viewport region.")
            return {'CANCELLED'}
        views = load_views()
        views[self.view_name] = {
            "location": list(region.view_location),
            "rotation": list(region.view_rotation),
            "distance": region.view_distance,
            "perspective": region.view_perspective,
            "focal_length": space.lens if space else 50.0,
        }
        save_views(views)
        self.report({'INFO'}, f"View '{self.view_name}' saved.")
        return {'FINISHED'}

    def invoke(self, context, event):
        self.view_name = "New View"
        return context.window_manager.invoke_props_dialog(self)

class BONSAI_OT_restore_view(bpy.types.Operator):
    bl_idname = "bonsai.restore_view"
    bl_label = "Restore View"

    view_name: bpy.props.StringProperty()

    def execute(self, context):
        region, space = get_region_and_space(context)
        views = load_views()
        data = views.get(self.view_name)
        if not (region and data):
            self.report({'ERROR'}, f"View '{self.view_name}' not found.")
            return {'CANCELLED'}

        region.view_location = data["location"]
        region.view_rotation = data["rotation"]
        region.view_distance = data["distance"]
        region.view_perspective = data.get("perspective", 'PERSP')
        if space and "focal_length" in data:
            space.lens = data["focal_length"]

        self.report({'INFO'}, f"View '{self.view_name}' restored.")
        return {'FINISHED'}

class BONSAI_OT_update_view(bpy.types.Operator):
    bl_idname = "bonsai.update_view"
    bl_label = "Update View"

    view_name: bpy.props.StringProperty()

    def execute(self, context):
        region, space = get_region_and_space(context)
        views = load_views()
        if self.view_name not in views:
            self.report({'ERROR'}, f"View '{self.view_name}' not found.")
            return {'CANCELLED'}

        views[self.view_name] = {
            "location": list(region.view_location),
            "rotation": list(region.view_rotation),
            "distance": region.view_distance,
            "perspective": region.view_perspective,
            "focal_length": space.lens if space else 50.0,
        }
        save_views(views)
        self.report({'INFO'}, f"View '{self.view_name}' updated.")
        return {'FINISHED'}

class BONSAI_OT_rename_view(bpy.types.Operator):
    bl_idname = "bonsai.rename_view"
    bl_label = "Rename View"

    old_name: bpy.props.StringProperty()
    new_name: bpy.props.StringProperty(name="New Name")

    def execute(self, context):
        views = load_views()
        if self.old_name in views:
            views[self.new_name] = views.pop(self.old_name)
            save_views(views)
            self.report({'INFO'}, f"Renamed '{self.old_name}' to '{self.new_name}'.")
            return {'FINISHED'}
        else:
            self.report({'ERROR'}, f"View '{self.old_name}' not found.")
            return {'CANCELLED'}

    def invoke(self, context, event):
        self.new_name = self.old_name
        return context.window_manager.invoke_props_dialog(self)

class BONSAI_OT_delete_view(bpy.types.Operator):
    bl_idname = "bonsai.delete_view"
    bl_label = "Delete View"

    view_name: bpy.props.StringProperty()

    def execute(self, context):
        views = load_views()
        if self.view_name in views:
            del views[self.view_name]
            save_views(views)
            self.report({'INFO'}, f"View '{self.view_name}' deleted.")
            return {'FINISHED'}
        else:
            self.report({'ERROR'}, f"View '{self.view_name}' not found.")
            return {'CANCELLED'}

class BONSAI_OT_save_views_as_json(bpy.types.Operator, ExportHelper):
    bl_idname = "bonsai.save_views_as_json"
    bl_label = "Save Viewport Bookmarks to File"
    filename_ext = ".json"

    def execute(self, context):
        full_data = {"type": "bonsai_viewport_bookmarks", "views": _cached_views}
        try:
            with open(self.filepath, "w", encoding="utf-8") as f:
                json.dump(full_data, f, indent=2)
            self.report({'INFO'}, "Viewport bookmarks saved successfully.")
        except Exception as e:
            self.report({'ERROR'}, f"Error saving bookmarks: {e}")
        return {'FINISHED'}

class BONSAI_OT_load_views_from_json(bpy.types.Operator, ImportHelper):
    bl_idname = "bonsai.load_views_from_json"
    bl_label = "Load Viewport Bookmarks from File"
    filename_ext = ".json"

    def execute(self, context):
        global _cached_views
        try:
            with open(self.filepath, "r", encoding="utf-8") as f:
                data = json.load(f)
                if isinstance(data, dict) and data.get("type") == "bonsai_viewport_bookmarks":
                    _cached_views.clear()
                    _cached_views.update(data.get("views", {}))
                    save_views(_cached_views)
                    self.report({'INFO'}, "Viewport bookmarks loaded successfully.")
                    return {'FINISHED'}
            self.report({'ERROR'}, "Invalid viewport bookmarks file.")
            return {'CANCELLED'}
        except Exception as e:
            self.report({'ERROR'}, f"Error loading bookmarks: {e}")
            return {'CANCELLED'}

class BONSAI_PT_viewport_bookmarks_panel(bpy.types.Panel):
    bl_label = "Bonsai Viewport Bookmarks"
    bl_idname = "BONSAI_PT_viewport_bookmarks_panel"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Bonsai"

    def draw(self, context):
        layout = self.layout
        layout.operator("bonsai.save_current_view", icon="ADD")
        layout.operator("bonsai.save_views_as_json", icon="EXPORT")
        layout.operator("bonsai.load_views_from_json", icon="IMPORT")
        layout.separator()

        views = _cached_views
        if views:
            layout.label(text="Saved Views:")
            for name in sorted(views.keys()):
                row = layout.row(align=True)
                op = row.operator("bonsai.restore_view", text=name, icon='VIEW3D')
                op.view_name = name
                row.operator("bonsai.update_view", text="", icon="FILE_REFRESH").view_name = name
                row.operator("bonsai.rename_view", text="", icon="GREASEPENCIL").old_name = name
                row.operator("bonsai.delete_view", text="", icon="TRASH").view_name = name
        else:
            layout.label(text="No saved views.")

# ---------------------- Registration ----------------------

classes = [
    BONSAI_OT_save_current_view,
    BONSAI_OT_restore_view,
    BONSAI_OT_update_view,
    BONSAI_OT_rename_view,
    BONSAI_OT_delete_view,
    BONSAI_OT_save_views_as_json,
    BONSAI_OT_load_views_from_json,
    BONSAI_PT_viewport_bookmarks_panel,
]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)

if __name__ == "__main__":
    register()
